@@ -3,7 +3,7 @@ import cStringIO, struct, collections, socket
 from rtypes import *
 from misc import FunctionMapper
 from rexceptions import RResponseError, REvalError
-from taggedContainers import TaggedList, asTaggedArray
+from taggedContainers import TaggedList, asTaggedArray, asAttrArray
 
 DEBUG = 0
 
@@ -327,7 +327,7 @@ class RParser(object):
 
     @fmap(XT_ARRAY_BOOL, XT_ARRAY_INT, XT_ARRAY_DOUBLE, XT_ARRAY_STR)
     def xt_array(self, lexeme):
-        data = self._nextExprData(lexeme)
+        data = self._nextExprData(lexeme)  # converts lexeme into a numpy array
         if lexeme.hasAttr and lexeme.attrTypeCode == XT_LIST_TAG:
             for tag, value in lexeme.attr:
                 if tag == 'dim':
@@ -336,10 +336,18 @@ class RParser(object):
                 elif tag == 'names':
                     # convert numpy-vector 'value' into list to make taggedarray work properly:
                     data = asTaggedArray(data, list(value))
-                elif tag in ['dimnames', 'assign']:
-                    print 'Warning: applying LIST_TAG "%s" on array not yet implemented' % tag
                 else:
-                    raise NotImplementedError('cannot apply tag "%s" on array' % tag)
+                    # there are unknow tags in the attribute, just collect them in a dictionary
+                    # of the array. (if array is a plain numpy array we need to convert it into
+                    # a view to be able to store new attributes to it):
+                    try:
+                        data.attr[tag] = value
+                    except AttributeError:
+                        data = asAttrArray(data, {tag: value})
+                        # make a subclass of the array class:
+                        #class AttrArray(_AttrArray, data.__class__): pass
+                        #data = data.view(AttrArray)
+                        #data.attr = {tag: value}
         return data
 
     @fmap(XT_VECTOR, XT_LANG_NOTAG, XT_LIST_NOTAG)
@@ -403,6 +411,25 @@ def rparse(src, atomicArray=False):
 
 ########################################################################################
 
+class _AttrArray(object):
+    '''
+    This class should not be used directly, but rather be used as a mixin class
+    when subclassing from a numpy class.
+    e.g.  
+    class RAttrClass(numpy.ndarray, AttrArray): 
+        pass
+    
+    Such a sub-class can be used to to store additional information in extra
+    attributes. (pure numpy arrays won't allow to create those).
+    '''
+    def __repr__(self):
+        r = super(_AttrArray, self).__repr__()
+        if hasattr(self, 'attr'):
+            return r[:-1] + ', attr=' + repr(self.attr) + ')'
+        return r
+
+    
+
 class Closure(object):
     'Very simple container to return "something" for a closure. Not really usable in Python though.'
     def __init__(self, lexeme, aList1, aList2):
