@@ -3,13 +3,38 @@ import numpy
 
 class TaggedList(object):
     # This code is mainly based on UserList.UserList and modified for tags
+    '''
+    A tagged list is useful for additionally addressing individual items by 
+    name instead of only by index. In contrast to dictionaries multiple items
+    can have the same name or key. However only the first one will be found.
+    
+    In many cases a TaggedList behaves like a normal list, however for lazyness
+    reasons of the programmer not all methods are implemented yet.
+    
+    Example:
+    l = TaggedList( [('v1', 1), ('v2', 2), 3, ('v2', 4)] )
+    l[0]     # returns 1
+    l['v1']  # returns 1
+    l['v2']  # returns 2  (not 4 !)
+    l[3]     # returns 4
+    
+    Data can be appended or inserted in the following way:
+    l.insert(0, x=3)
+    l['x']   # returns 3
+    l[0]     # also returns 3
+    
+    l.append(y=3)
+    l[-1]    # returns 3
+    '''
     def __init__(self, initlist=[]):
+        '''
+        Items in initlist can either be 
+        - tuples of (key,values)
+        - or plain values
+        Keys can be None or empty strings in item tuples.
+        '''
         self.values = []
         self.keys = []
-        # Items in initlist can either be 
-        # - tuples of (key,values)
-        # - or plain values
-        # Keys can be None of empty strings in item tuples.
         for idx, item in enumerate(initlist):
             try:
                 key, value = item
@@ -21,15 +46,13 @@ class TaggedList(object):
                 self.values.append(value)
                 self.keys.append(key)
                 
-    @property
     def astuples(self):
         'converts a TaggedList into a representation suitable to be provided to __init__()'
         return zip(self.keys, self.values)
         
     def __repr__(self): 
-        data = ["%s='%s'" % (key, value) if key else "'%s'" % value for key,value in self.astuples]
+        data = ["%s=%s" % (key, repr(value)) if key else "'%s'" % value for key,value in self.astuples()]
         return '<TaggedList(%s)>' % ', '.join(data)
-        return repr(self.astuples)
 
 #    def __lt__(self, other): return self.values <  self.__cast(other)
 #    def __le__(self, other): return self.values <= self.__cast(other)
@@ -68,7 +91,7 @@ class TaggedList(object):
 
     def __getslice__(self, i, j):
         i = max(i, 0); j = max(j, 0)
-        return self.__class__(self.astuples[i:j])
+        return self.__class__(self.astuples()[i:j])
         
 #    def __setslice__(self, i, j, other):
 #        i = max(i, 0); j = max(j, 0)
@@ -136,6 +159,7 @@ class TaggedList(object):
     def insert(self, i, *value, **key_and_value):
         if len(value)==1 and not key_and_value:
             key = None
+            value = value[0]
         elif len(key_and_value)==1 and not value:
             [(key, value)] = key_and_value.items()
         else:
@@ -177,16 +201,28 @@ class TaggedList(object):
 
 
 class AttrArray(numpy.ndarray):
-    'numpy.ndarray with additional "attr"-attribute'
+    'numpy.ndarray with additional "attr"-container'
     attr = None
     
+    def __repr__(self):
+        r = super(AttrArray, self).__repr__()
+        if hasattr(self, 'attr'):
+            return r[:-1] + ', attr=' + repr(self.attr) + ')'
+        return r
+
 def asAttrArray(ndarray, attr):
     arr = ndarray.view(AttrArray)
     arr.attr = attr
-    return attr
-    
+    return arr
+
 class TaggedArray(AttrArray):
     attr = []
+    def __repr__(self):
+        r = super(AttrArray, self).__repr__()
+        if hasattr(self, 'attr'):
+            return r[:-1] + ', key=' + repr(self.attr) + ')'
+        return r
+
     def __getitem__(self, idx_or_name):
         try:
             return numpy.ndarray.__getitem__(self, idx_or_name)
@@ -195,14 +231,14 @@ class TaggedArray(AttrArray):
         try:
             return numpy.ndarray.__getitem__(self, self.attr.index(idx_or_name))
         except ValueError:
-            raise KeyError('No tag "%s" available for array' % idx_or_name)
+            raise KeyError('No key "%s" available for array' % idx_or_name)
             
     def keys(self):
         return self.attr[:]
 
 def asTaggedArray(ndarray, tags):
     if len(tags) != len(ndarray):
-        raise ValueError('Number of tags must match size of array')
+        raise ValueError('Number of keys must match size of array')
     arr = ndarray.view(TaggedArray)
     arr.attr = tags
     return arr
